import machine, os, sdcard
import time
import ntptime
from machine import Pin, SPI
import network

from onewire import OneWire
from ds18x20 import DS18X20

#Dateiname für die CSV-Datei
outputFilename = 'ds18b20.csv'

#Zugangsdaten zum lokalen WiFi-Netzwerk
ssid = 'HOTSPOT'
password = '123456789'

#Zeit welche zwischen zwei Lesevorgängen
#vergehen muss in Sekunden.
sleepBetweenMeasurements = 5

#Starten des Netzwerkes im Station Mode
sta= network.WLAN(network.STA_IF)
sta.active(True)

#der Temperatursensor DS18B20 ist am GPIO 4 angeschlossen
sensorPin = Pin(4)
sensor = DS18X20(OneWire(sensorPin))
sensors = ()

#CS Pin mit GP2 als Ausgang 
cs = Pin(2, Pin.OUT)
#Initialisieren der SPI Kommunikation
spi = SPI(2, baudrate=1000000, polarity=0, phase=0, bits=8,
          firstbit=SPI.MSB, sck=Pin(18), mosi=Pin(23), miso=Pin(19))
#Initialisieren des SD-Card Objektes mit
#der konfigurierten SPI Schnittstelle
sd = sdcard.SDCard(spi, cs)


def setup():
    global sensors
    #mounten / einbinden der SD-Karte
    os.mount(sd, '/sd')
    #einen Zugriffsversuch starten
    #sollte dieser misslingen so wird eine Exception geworfen
    os.listdir('/sd')
    
    #ermitteln der angeschlossenen Sensoren
    sensors = sensor.scan()
    #ausgeben der Anzahl der Sensoren
    print('gefundene DS18B20 Sensoren: ', len(sensors))
    
    #aufbauen der WiFi-Verbindung
    doConnect()
    

def writeLine(timestampValue, ds18b20Value):
    #öffnen der Datei ds18b20.csv mit dem Parameter "a"
    #für schreibzugriff mit anhängen (append) 
    file = open('/sd/'+outputFilename, 'a')
    #schreiben einer Zeile
    file.write("\"{timestamp}\";{value}\r\n".format(timestamp=timestampValue, value=ds18b20Value))
    #schließen der Datei
    file.close()

def doConnect():
    #Wenn die Verbindung nicht erstellt wurde, dann...
    if not sta.isconnected():
        print("Aufbau der WiFi-Verbindung zu ", ssid)
        #Aufbau der WiFi-Verbindung mit den Daten
        sta.connect(ssid, password)
        #solange die Verbindung noch nicht hergestellt wurde
        #dann soll diese Schleife laufen
        while not sta.isconnected():
            pass
    #Wenn die WiFi-Verbindung erfolgreich aufgebaut wurde,
    #dann soll auf der seriellen Schnittstelle der Text ausgegeben werden
    print("Verbindung erfolgreich zu {ssid} aufgebaut!".format(ssid=ssid))

def readTimestamp():
    #setzen der Zeit auf dem Mikrocontroller
    ntptime.settime()
    #auslesen der Zeit in ein Tupel
    timeTpl = time.gmtime()
    #Formatieren der Daten in ein deutsches Datum/Zeit Format
    return "{day}.{month}.{year} {hour}:{minute}:{seconds}".format(day = timeTpl[2], month = timeTpl[1], year = timeTpl[0], hour = timeTpl[3]+2, minute = timeTpl[4], seconds = timeTpl[5])
    
def readSensorValue():
    sensor.convert_temp()
    #nach dem Aufruf der Funktion convert_temp()
    #muss man min. 750 Millisekunden warten
    time.sleep(0.750)
    return round(sensor.read_temp(sensors[0]),2)
    

def main():
    #starten der Endlosschleife
    while True:
       #lesen des Zeitstempels vom NTP-Server
       timestampValue = readTimestamp()
       #lesen der Temperatur vom Sensor
       ds18b20Value = readSensorValue()
       #Ausgeben der Daten auf der seriellen Schnittstelle
       print("{timestamp} - {tempValue}".format(timestamp=timestampValue, tempValue=ds18b20Value))
       #Speichern der Daten in der CSV-Datei auf der SD-Karte
       writeLine(timestampValue, ds18b20Value)
       #eine kleine Pause einlegen
       time.sleep(sleepBetweenMeasurements)

setup()
main()
